/*
 * © NHN Corp. All rights reserved.
 * NHN Corp. PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.toast.android.gamebase.sample.ui;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.ActivityNotFoundException;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.net.Uri;
import android.os.Handler;
import android.os.Looper;
import android.widget.Switch;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.toast.android.gamebase.Gamebase;
import com.toast.android.gamebase.GamebaseDataCallback;
import com.toast.android.gamebase.auth.data.AuthToken;
import com.toast.android.gamebase.auth.data.GraceBanInfo;
import com.toast.android.gamebase.auth.data.TemporaryWithdrawalInfo;
import com.toast.android.gamebase.auth.mapping.data.ForcingMappingTicket;
import com.toast.android.gamebase.base.GamebaseException;
import com.toast.android.gamebase.base.log.Logger;
import com.toast.android.gamebase.base.push.PushConfiguration;
import com.toast.android.gamebase.sample.GamebaseManager;

import java.net.URLDecoder;
import java.util.Locale;

public class GameCodeManager {
    private static final String PREFERENCE_NAME = "gamebase.sample.application";
    private static final String PREF_KEY_PUSH_CONFIGURATION = "gamebase.sample.pref.push.configuration";

    public static void savePushConfiguration(@NonNull final Context context,
                                             @Nullable final PushConfiguration pushConfiguration) {
        if (pushConfiguration == null) {
            return;
        }
        SharedPreferences pref = context.getSharedPreferences(PREFERENCE_NAME, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = pref.edit();
        editor.putString(PREF_KEY_PUSH_CONFIGURATION, pushConfiguration.toJsonString());
        editor.apply();
    }

    public static PushConfiguration loadPushConfiguration(@NonNull final Context context) {
        SharedPreferences pref = context.getSharedPreferences(PREFERENCE_NAME, Context.MODE_PRIVATE);
        if (pref.contains(PREF_KEY_PUSH_CONFIGURATION)) {
            String pushConfigStr = pref.getString(PREF_KEY_PUSH_CONFIGURATION, "{}");

            try {
                return PushConfiguration.from(pushConfigStr);
            } catch (Exception e) {
                e.printStackTrace();
                Logger.w("PushConfiguration",
                        "PushConfiguration.from(" + pushConfigStr + ")\n : " + e.getMessage());
                return null;
            }
        }

        return null;
    }

    public static void showErrorAndReturnToTitle(final Activity activity, final String title, final String message) {
        final DialogInterface.OnClickListener returnToTitle = (dialog, which) -> returnToTitle(activity);
        Gamebase.Util.showAlert(activity, title, message, returnToTitle);
    }

    public static void showUnregisteredVersionAndMoveToStore(final Activity activity, final String updateUrl, final String message) {
        final int requestCode = 123;
        final DialogInterface.OnClickListener moveToStore = (dialog, which) -> {
            try {
                Intent marketIntent = new Intent(Intent.ACTION_VIEW);
                marketIntent.setData(Uri.parse(updateUrl));

                activity.startActivityForResult(marketIntent, requestCode);
            } catch (ActivityNotFoundException ignored) {
            }
        };
        Gamebase.Util.showAlert(activity, "Unregistered Game Version", message, moveToStore);
    }

    public static void pauseGame() {}

    public static void returnToTitle(final Activity activity) {
        activity.finish();
    }

    public static void showError(final Activity activity, final String title, final String message) {
        Gamebase.Util.showAlert(activity, title, message);
    }

    public static void moveToLoginScene(final Activity activity) {
        SplashActivity.LoadLoginActivity(activity);
    }

    public static void returnToLoginScene(final Activity activity) {
        try {
            ((MainActivity) activity).loadLoginFragment();
        } catch (Exception ignored) {
        }
    }

    public static void moveToInGameStore(final Activity activity) {
        try {
            ((MainActivity) activity).loadPurchaseFragment();
        } catch (Exception ignored) {
        }
    }

    public static void moveToPushSettingsMenu(final Activity activity) {
        try {
            ((MainActivity) activity).loadPushFragment();
        } catch (Exception ignored) {
        }
    }

    public static void startGame(final Activity activity) {
        ((MainActivity) activity).loadApiListFragment();
    }

    public static void addMappingAndChangeAccountUI(final Activity activity, final String providerName, final Switch mappingSwitch) {
        GamebaseManager.addMapping(activity, providerName, (authToken, exception) -> {
            if (Gamebase.isSuccess(exception)) {
                ((MainActivity) activity).reloadMappingFragment();
            } else {
                changeAccountSwitchUI(activity, mappingSwitch, false);
            }
        });
    }

    public static void removeMappingAndChangeAccountUI(final Activity activity, final String providerName, final Switch mappingSwitch) {
        GamebaseManager.removeMapping(activity, providerName, exception -> {
            if (Gamebase.isSuccess(exception)) {
                // succeeded
            } else {
                changeAccountSwitchUI(activity, mappingSwitch, true);
            }
        });
    }

    private static void changeAccountSwitchUI(final Activity activity, final Switch changeSwitch, final boolean flag) {
        activity.runOnUiThread(() -> changeSwitch.setChecked(flag));
    }

    public static void moveToAddMappingForciblyFlow(final Activity activity,
                                                    final GamebaseException addMappingException,
                                                    final GamebaseDataCallback<AuthToken> callback) {
        final ForcingMappingTicket forcingMappingTicket = ForcingMappingTicket.from(addMappingException);
        final String mappedUserId = forcingMappingTicket.mappedUserId;
        activity.runOnUiThread(() -> new AlertDialog.Builder(activity)
                .setTitle("Do you want overwrite your account?")
                .setMessage("There is a account that you want to mapping.\n" +
                        "Do you want to erase your account\n\n" + mappedUserId + "\n\n??")
                .setPositiveButton("Erase '" + mappedUserId + "'", (dialog, which) -> {
                    activity.runOnUiThread(() -> new AlertDialog.Builder(activity)
                            .setTitle("CAUTION")
                            .setMessage("The account '"+ mappedUserId + "' will be erased!\n" +
                                    "Are you sure you want to continue?")
                            .setPositiveButton("YES!!!", (dialog13, which12) -> {
                                GamebaseManager.addMappingForcibly(activity, forcingMappingTicket,
                                        (authToken, addMappingForciblyException) -> {
                                            if (callback != null) callback.onCallback(authToken, addMappingForciblyException);
                                        });
                            })
                            .setNegativeButton("No.", (dialog12, which1) -> {
                                if (callback != null) callback.onCallback(null, addMappingException);
                            })
                            .setCancelable(true)
                            .setOnCancelListener(dialog1 -> {
                                if (callback != null) callback.onCallback(null, addMappingException);
                            })
                            .show());
                })
                .setNegativeButton("Login to '" + mappedUserId + "'", (dialog, which) -> {
                    GamebaseManager.changeLogin(activity, forcingMappingTicket, (authToken, changeLoginException) -> {
                        if (Gamebase.isSuccess(changeLoginException)) {
                            GamebaseManager.onLoginSucceeded(activity, authToken);
                        }
                    });
                })
                .setCancelable(true)
                .setOnCancelListener(dialog -> {
                    if (callback != null) callback.onCallback(null, addMappingException);
                })
                .show());
    }

    public static void showTemporaryWithdrawUI(final Activity activity,
                                               final TemporaryWithdrawalInfo temporaryWithdrawalInfo,
                                               final GamebaseManager.Callback callback) {
        if (activity == null || temporaryWithdrawalInfo == null) {
            if (callback != null) callback.onCallback();
            return;
        }
        new Handler(Looper.getMainLooper()).post(() -> {
            String date = new java.text.SimpleDateFormat("MM/dd/yyyy HH:mm:ss", Locale.getDefault())
                    .format(new java.util.Date(temporaryWithdrawalInfo.getGracePeriodDate()));
            new AlertDialog.Builder(activity)
                    .setTitle("This account will be erased!")
                    .setMessage("You can login until : " + date)
                    .setPositiveButton("OK", new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialog, int which) {
                            if (callback != null) callback.onCallback();
                        }
                    })
                    .setCancelable(false)
                    .show();
        });
    }

    public static void showGraceBanUI(final Activity activity,
                                      final GraceBanInfo graceBanInfo,
                                      final GamebaseManager.Callback callback) {
        if (activity == null || graceBanInfo == null) {
            if (callback != null) callback.onCallback();
            return;
        }
        new Handler(Looper.getMainLooper()).post(() -> {
            String periodDate = new java.text.SimpleDateFormat("MM/dd/yyyy HH:mm:ss", Locale.getDefault())
                    .format(new java.util.Date(graceBanInfo.getGracePeriodDate()));
            String message;
            try {
                message = URLDecoder.decode(graceBanInfo.getMessage(), "utf-8");
            } catch (Exception e) {
                message = e.getMessage();
            }
            message += "\n\n" +
                    "Period Date : " + periodDate + "\n\n " +
                    "* Release Condition * \n" +
                    "Pay " + graceBanInfo.getReleaseRuleCondition().getAmount() +
                    graceBanInfo.getReleaseRuleCondition().getCurrency() +
                    "\n" + graceBanInfo.getReleaseRuleCondition().getConditionType() + "\n" +
                    "Pay " + graceBanInfo.getReleaseRuleCondition().getCount() + " time(s) more.\n\n" +
                    "* Your payment info * \n" +
                    "Paid " + graceBanInfo.getPaymentStatus().getAmount() +
                    graceBanInfo.getPaymentStatus().getCurrency() + "\n" +
                    "Paid " + graceBanInfo.getPaymentStatus().getCount() + " time(s).";
            new AlertDialog.Builder(activity)
                    .setTitle("This account will be banned!")
                    .setMessage(message)
                    .setPositiveButton("OK", (dialog, which) -> {
                        if (callback != null) callback.onCallback();
                    })
                    .setCancelable(false)
                    .show();
        });
    }
}
